/*
 * osubcast.c
 * Send an OSU network management broadcast to locate devices.
 * Reverse-engineered from watching KarlNet's configurator.
 * (i.e. I don't know *why* it works =)
 *
 * Bill Fenner <fenner@research.att.com>
 * 13 March 2000
 *
 * $Id: osubcast.c,v 1.1 2000/03/13 19:10:03 fenner Exp $
 */
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <netinet/in.h>

int
main(int argc, char **argv)
{
	/* want to do siocgifconf and send to all up broadcast addrs */
	int s;
	struct sockaddr_in myside;
	struct sockaddr_in dst;
	unsigned char buf[200];
	struct timeval fin, tmo;
	int one = 1;

	if (argc < 2) {
		fprintf(stderr, "usage: %s broadcast-address\n", argv[0]);
		exit(1);
	}
	if ((s = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
		perror("socket");
		exit(1);
	}
	myside.sin_family = AF_INET;
	myside.sin_len = sizeof(myside);
	myside.sin_addr.s_addr = INADDR_ANY;
	myside.sin_port = 0;
	if (bind(s, (struct sockaddr *)&myside, myside.sin_len) < 0) {
		perror("bind");
		exit(1);
	}
	if (setsockopt(s, SOL_SOCKET, SO_BROADCAST, &one, sizeof(one)) < 0) {
		perror("SO_BROADCAST");
		exit(1);
	}
	dst.sin_family = AF_INET;
	dst.sin_len = sizeof(dst);
	dst.sin_addr.s_addr = inet_addr(argv[1]);
	dst.sin_port = htons(192);	/* OSU Network Monitoring System */
	bzero(buf, sizeof(buf));
	buf[0] = 1;
	if (sendto(s, buf, 116, 0, (struct sockaddr *)&dst, dst.sin_len) < 0) {
		perror("sendto");
		exit(1);
	}
	gettimeofday(&fin, NULL);
	fin.tv_sec += 5;	/* XXX */
	while (1) {
		fd_set r;

		gettimeofday(&tmo, NULL);
		tmo.tv_sec = fin.tv_sec - tmo.tv_sec;
		tmo.tv_usec = fin.tv_usec - tmo.tv_usec;
		if (tmo.tv_usec < 0) {
			tmo.tv_sec--;
			tmo.tv_usec += 1000000;
		}
		if (tmo.tv_sec < 0) {
			break;
		}
		FD_ZERO(&r);
		FD_SET(s, &r);
		select(s + 1, &r, NULL, NULL, &tmo);	/*XXX*/
		if (FD_ISSET(s, &r)) {
			struct sockaddr_in from;
			int fromlen = sizeof(from);
			int len;
			int i;

			if ((len = recvfrom(s, buf, sizeof(buf), 0,
				    (struct sockaddr *)&from, &fromlen)) < 0) {
				perror("recvfrom");
				continue;
			}
			printf("%s|", inet_ntoa(from.sin_addr));
			if (len != 116) {
				printf("weird len %d\n", len);
				continue;
			}
			/*XXX trusting them to nul-terminate strings... */
			printf("%s|%s", &buf[0x30], &buf[0x54]);
			for (i = 0x24; i < 0x2a; i++)
				printf("%c%x", (i == 0x24) ? '|':':', buf[i]);
			printf("\n");
		}
	}
}
